import matplotlib
import matplotlib.colors as colors
import matplotlib.pyplot as plt
import numpy as np
import matplotlib.gridspec as gridspec
import svgutils
import svgutils.transform as sg

tex_settings = {
    "text.usetex": True,
    "font.family": "lmr",
    "font.serif": "Computer Modern Roman",
    "text.latex.preamble": r"\usepackage{siunitx} \usepackage[version=4]{mhchem} \sisetup{detect-all}",
    "font.size": 10,
    "axes.labelsize": 12,
    "xtick.labelsize": 12,
    "ytick.labelsize": 12
}
matplotlib.rcParams.update(tex_settings)

linewidth = 6.13899
color_list = ["#5790fc", "#f89c20", "#e42536", "#964a8b", "#9c9ca1", "#7a21dd"]

### Figure 1

csd_2dot = np.load("../codes/data/car_ect_crossover.npz", allow_pickle=True)
u_scaling = np.load("../codes/data/u_scaling.npz", allow_pickle=True)

fig = plt.figure(figsize=(linewidth * 0.95, 4))
gs = gridspec.GridSpec(2, 16, figure=fig, hspace=0.5, wspace=45)
fig.tight_layout()
ax1 = fig.add_subplot(gs[0, :10])
ax1.axis('off')
ax2 = fig.add_subplot(gs[0, 10:])
ax3 = fig.add_subplot(gs[1, :5])
ax4 = fig.add_subplot(gs[1, 5:10], sharey=ax3)
ax5 = fig.add_subplot(gs[1, 10:15], sharey=ax3)
plt.setp(ax4.get_yticklabels(), visible=False)
plt.setp(ax5.get_yticklabels(), visible=False)

for i, axs in enumerate([ax3,ax4,ax5]):
    im = axs.pcolormesh(
        csd_2dot["eps_arr"],
        csd_2dot["eps_arr"],
        csd_2dot["csd"][i],
        cmap="seismic",
        vmin=-3,
        vmax=3,
        rasterized=True
        )
    axs.set_xticks([-4,-2,0,2])
    axs.set_yticks([-4,-2,0,2])
    axs.set_xlabel(r"$\epsilon_R/t$")
ax3.set_ylabel(r"$\epsilon_L/t$")

ax3.text(-6.4, 3, '(d)', size=12, weight="semibold")
ax4.text(-6.4, 3, '(e)', size=12, weight="semibold")
ax5.text(-6.4, 3, '(f)', size=12, weight="semibold")

ax2.plot(u_scaling["U_arr"], u_scaling["sweet_spot_arr"][:, 0], color=color_list[0], label=r"$\epsilon^*/t$")
ax2.plot(u_scaling["U_arr"], u_scaling["sweet_spot_arr"][:, 1], color=color_list[1], label=r"$\Delta^*/t$")
ax2.hlines(1/np.sqrt(2), 0, 100, "k", ls="--")
ax2.set_xlabel(r"$U/t$")
ax2.legend(loc=1, prop={'size': 8})
ax2.set_xlim(0,100)
ax2.set_ylim(0,1.2)
ax2.set_yticks([0, 1./np.sqrt(2), 1], [r"$0$", r"$\frac{1}{\sqrt{2}}$", r"$1$"])
fig.text(0.605, 0.88, '(c)', size=12, weight="semibold")
fig.text(0.085, 0.88, '(a)', size=12, weight="semibold")
fig.text(0.35, 0.88, '(b)', size=12, weight="semibold")


cax = fig.add_axes([ax5.get_position().x1+0.01,ax5.get_position().y0,0.02,ax5.get_position().height])
cbar = plt.colorbar(im, cax=cax)
cbar.ax.set_title(r'$\delta E/t$')

ax1.remove()

plt.savefig(
    "../publication/figures/car_ect_crossover.svg", bbox_inches='tight', transparent=True, dpi=500
    )


figures_main_path = '../publication/figures/'

figures_paths = [
    figures_main_path+'sketch-2dots.svg',
    figures_main_path+'sketch-csd.svg',
    figures_main_path+'car_ect_crossover.svg',
]
# load matpotlib-generated figures

figs = [sg.fromfile(path) for path in figures_paths]

# get the plot objects
plots = [fig.getroot() for fig in figs]
wspace = 0.2
hspace = 0.1

plots[0].moveto(0.1, 0.3 + hspace, scale_x=0.05*0.42)
plots[1].moveto(0.7 + 6*wspace, 0.3 + hspace, scale_x=0.05*0.42)
plots[2].moveto(0, hspace, scale_x=0.05*0.3)
fig1 = svgutils.compose.Figure(str(linewidth)+"in", "4.2in", *plots)

fig1.save(figures_main_path+'Figure1.svg')

### Figure 2

fig, axs = plt.subplots(nrows=2,ncols=3, sharey="row", figsize=(linewidth*0.95,4))
fig.tight_layout()
fig.subplots_adjust(hspace=0.4)

for i in range(3):
    im1 = axs[0, i].pcolormesh(csd_2dot["eps_arr"], csd_2dot["eps_arr"], csd_2dot["csd_per_U"][i], cmap="seismic", vmin=-3, vmax=3, rasterized=True
)
    axs[0, i].set_xticks([-4,-2,0,2])
    axs[0, i].set_yticks([-4,-2,0,2])
    axs[0, i].set_xlabel(r"$\epsilon_R/t$")

    im2 = axs[1, i].pcolormesh(csd_2dot["bias_params"][()]["dmu"], csd_2dot["bias_params"][()]["Vbias"], csd_2dot["cond_per_U"][i][:,:,0].T, cmap="gist_heat_r", vmin=0, rasterized=True)
    axs[1, i].set_xticks([-2,0,2])
    axs[1, i].set_yticks([-2,0,2])
    axs[1, i].set_xlabel(r"$\delta\epsilon_R/t$")

axs[0, 0].set_ylabel(r"$\epsilon_L/t$")
axs[1, 0].set_ylabel(r"$eV_\text{bias}/t$")

cax1 = fig.add_axes([axs[0, 2].get_position().x1+0.02,axs[0, 2].get_position().y0,0.02,axs[0, 2].get_position().height])
cbar1 = plt.colorbar(im1, cax=cax1)
cbar1.ax.set_title(r'$\delta E/t$')

cax2 = fig.add_axes([axs[1, 2].get_position().x1+0.02,axs[1, 2].get_position().y0,0.02,axs[1, 2].get_position().height])
cbar2 = plt.colorbar(im2, cax=cax2)
cbar2.ax.set_title(r'$G_{LL} [e^2/h]$')

axs[0, 0].text(-6.4,3, '(a)', size=12, weight="semibold")
axs[0, 1].text(-6.4,3, '(b)', size=12, weight="semibold")
axs[0, 2].text(-6.4,3, '(c)', size=12, weight="semibold")
axs[1, 0].text(-4,3, '(d)', size=12, weight="semibold")
axs[1, 1].text(-4,3, '(e)', size=12, weight="semibold")
axs[1, 2].text(-4,3, '(f)', size=12, weight="semibold")

plt.savefig(
    "../publication/figures/u_scaling.svg", bbox_inches='tight', transparent=False, dpi=500
    )


### Figure 4

fig, ax = plt.subplots(figsize=(linewidth, 1.5))
ax.axis("off")
fig.text(0.1, 1.5, "(a)", size=12, weight="semibold")
fig.text(0.43, 1.5, "(b)", size=12, weight="semibold")
plt.savefig(
    "../publication/figures/three_site_fig.svg", bbox_inches='tight', transparent=True, dpi=500
    )

figures_main_path = '../publication/figures/'

figures_paths = [figures_main_path+'three_site_fig.svg',
figures_main_path+'sketch-dot-test.svg',
figures_main_path+'sketch-3dots.svg',
]
# load matpotlib-generated figures

figs = [sg.fromfile(path) for path in figures_paths]

# get the plot objects
plots = [fig.getroot() for fig in figs]
wspace = 0.2
hspace = 2

plots[0].moveto(-0.1,-0.1, scale_x=0.021)
plots[1].moveto(0.0,0.0, scale_x=0.022)
plots[2].moveto(0.0+3.05, 0., scale_x=0.022)
fig4 = svgutils.compose.Figure(str(linewidth)+"in", "1.45in", *plots)

fig4.save(figures_main_path+'Figure4.svg')

### Figure 5

three_dot = np.load("../codes/data/three_dot.npz", allow_pickle=True)

params_Bias = three_dot["bias_params"][()]
params_Bias_extended = three_dot["bias_params_x"][()]
dot_test = three_dot["dot_test"]
spectroscopy = three_dot["spectroscopy"]

fig, axs = plt.subplots(nrows=1, ncols=3, sharey="row", figsize=(linewidth * 0.95, 2))
fig.tight_layout()

for i in range(3):
    im1 = axs[i].pcolormesh(
        params_Bias["dmu"],
        params_Bias["Vbias"],
        dot_test[i].T,
        cmap="gist_heat_r",
        vmin=0,
        vmax=0.7,
        rasterized=True,
    )

axs[0].set_xlabel(r"$\epsilon_L/t$")
axs[1].set_xlabel(r"$\epsilon_R/t$")
axs[2].set_xlabel(r"$\epsilon_D/t$")
axs[0].set_ylabel(r"$eV_\text{bias}/t$")

cax1 = fig.add_axes(
    [
        axs[2].get_position().x1 + 0.02,
        axs[2].get_position().y0,
        0.02,
        axs[2].get_position().height,
    ]
)
cbar1 = plt.colorbar(im1, cax=cax1)
cbar1.ax.set_title(r"$G_{DD} [e^2/h]$")
axs[0].text(-3.1, 1, '(a)', size=12, weight="semibold")
axs[1].text(-2.8, 1, '(b)', size=12, weight="semibold")
axs[2].text(-2.8, 1, '(c)', size=12, weight="semibold")

plt.savefig(
    "../publication/figures/dot_test.svg", bbox_inches='tight', transparent=False, dpi=500
    )

### Figure 6

fig, axs = plt.subplots(nrows=1,ncols=3, sharey="row", figsize=(linewidth*0.95,2))
fig.tight_layout()

for i in range(3):
    im1 = axs[i].pcolormesh(
        params_Bias_extended["dmu"],
        params_Bias_extended["Vbias"],
        spectroscopy[i][:,:,3].T,
        cmap="gist_heat_r",
        vmin=0,
        vmax=0.7,
        rasterized=True
        )

axs[0].set_xlabel(r"$\epsilon_L/t$")
axs[1].set_xlabel(r"$\epsilon_M/t$")
axs[2].set_xlabel(r"$\epsilon_R/t$")
axs[0].set_ylabel(r"$eV_\text{bias}/t$")

cax1 = fig.add_axes(
    [
        axs[2].get_position().x1+0.02,
        axs[2].get_position().y0,0.02,
        axs[2].get_position().height
    ]
)
cbar1 = plt.colorbar(im1, cax=cax1)
cbar1.ax.set_title(r'$G_{RR} [e^2/h]$')

axs[0].text(-15.8, 1, '(a)', size=12, weight="semibold")
axs[1].text(-13.8, 1, '(b)', size=12, weight="semibold")
axs[2].text(-13.8, 1, '(c)', size=12, weight="semibold")

plt.savefig(
    "../publication/figures/no_scaling.svg", bbox_inches='tight', transparent=False, dpi=500
    )


## Appendix figures

### Figure 7

pmm_vs_pmmkp = np.load("../codes/data/pmm_vs_pmmkp.npz", allow_pickle=True)

params_Bias = pmm_vs_pmmkp["bias_params"][()]
csd_pmm = pmm_vs_pmmkp["csd_pmm"]
cond_pmm = pmm_vs_pmmkp["cond_pmm"]
csd_pmmkp = pmm_vs_pmmkp["csd"]
cond_pmmkp = pmm_vs_pmmkp["cond"]

cond_pmm_single = pmm_vs_pmmkp["cond_pmms"]
cond_pmmkp_single = pmm_vs_pmmkp["conds"]

fig = plt.figure(figsize=(linewidth * 0.95, 3.5))
plt.subplots_adjust(wspace=0.25)

outer = gridspec.GridSpec(1, 2, figure=fig, width_ratios=[1, 2])

gs1 = gridspec.GridSpecFromSubplotSpec(2, 1, subplot_spec=outer[0], hspace=0.15, wspace=0.15)
gs2 = gridspec.GridSpecFromSubplotSpec(
    2, 2, subplot_spec=outer[1], hspace=0.15, wspace=0.15
)

ax4 = fig.add_subplot(gs1[1])
ax5 = fig.add_subplot(gs2[1, 0])
ax6 = fig.add_subplot(gs2[1, 1], sharey=ax5)
ax2 = fig.add_subplot(gs2[0, 0], sharex=ax5)
ax3 = fig.add_subplot(gs2[0, 1], sharey=ax2, sharex=ax6)
ax1 = fig.add_subplot(gs1[0], sharex=ax4)

plt.setp(ax1.get_xticklabels(), visible=False)
plt.setp(ax2.get_xticklabels(), visible=False)
plt.setp(ax3.get_xticklabels(), visible=False)
plt.setp(ax3.get_yticklabels(), visible=False)
plt.setp(ax6.get_yticklabels(), visible=False)

im1=ax1.pcolormesh(params_Bias["dmu"], params_Bias["dmu"], csd_pmm, cmap="seismic", vmin=-1, vmax=1, rasterized=True)
ax1.set_ylabel(r"$\epsilon_L/t$", labelpad=-5)

cax1 = fig.add_axes(
    [
        ax1.get_position().x0,
        ax1.get_position().y1 +0.07,
        ax1.get_position().width,
        0.02
    ]
)
cbar1 = plt.colorbar(im1, cax=cax1, orientation="horizontal")
cbar1.ax.set_title(r'$\delta E/t$')

ax4.pcolormesh(params_Bias["dmu"], params_Bias["dmu"], csd_pmmkp, cmap="seismic", vmin=-1, vmax=1, rasterized=True)
ax4.set_xlabel(r"$\epsilon_R/t$")
ax4.set_ylabel(r"$\epsilon_L/t$", labelpad=-5)


im2=ax2.pcolormesh(params_Bias["dmu"], params_Bias["Vbias"], cond_pmm[:,:,0].T, cmap="gist_heat_r", vmin=0, vmax=1, rasterized=True)
ax2.set_ylabel(r"$eV_\text{bias}/t$", labelpad=-5)
ax2.set_yticks([-4,-2,0,2,4])

ax5.pcolormesh(params_Bias["dmu"], params_Bias["Vbias"], cond_pmmkp[:,:,0].T, cmap="gist_heat_r", vmin=0, vmax=1, rasterized=True)
ax5.set_xlabel(r"$\epsilon_L = \epsilon_R [t]$")
ax5.set_ylabel(r"$eV_\text{bias}/t$", labelpad=-5)
ax5.set_yticks([-4,-2,0,2,4])
ax5.set_xticks([-2,0,2])


cax2 = fig.add_axes(
    [
        ax2.get_position().x0,
        ax2.get_position().y1 + 0.07,
        ax2.get_position().width,
        0.02

    ]
)
cbar2 = plt.colorbar(im2, cax=cax2, orientation="horizontal")
cbar2.ax.set_title(r'$G_{LL} [e^2/h]$')
cbar2.set_ticklabels([r"$0$", r"$0.5$",r"$1$"])

im3=ax3.pcolormesh(params_Bias["dmu"], params_Bias["Vbias"], cond_pmm[:,:,1].T, cmap="seismic", vmin=-1, vmax=1, rasterized=True)

ax6.pcolormesh(params_Bias["dmu"], params_Bias["Vbias"], cond_pmmkp[:,:,1].T, cmap="seismic", vmin=-1, vmax=1, rasterized=True)
ax6.set_xlabel(r"$\epsilon_L = \epsilon_R [t]$")
ax6.set_xticks([-2,0,2])

cax3 = fig.add_axes(
    [
        ax3.get_position().x0,
        ax3.get_position().y1 + 0.07,
        ax3.get_position().width,
        0.02

    ]
)
cbar3 = plt.colorbar(im3, cax=cax3, orientation="horizontal")
cbar3.ax.set_title(r'$G_{LR} [e^2/h]$')
cbar3.set_ticklabels([r"$-1$", r"$0$", r"$1$"])

ax1.text(-4.3,1.9, '(a)', size=12, weight="semibold")
ax2.text(-2.3,3.1, '(b)', size=12, weight="semibold")
ax3.text(-2.3,3.1, '(c)', size=12, weight="semibold")
ax4.text(-4.3,1.9, '(d)', size=12, weight="semibold")
ax5.text(-2.3,3.1, '(e)', size=12, weight="semibold")
ax6.text(-2.3,3.1, '(f)', size=12, weight="semibold")

plt.savefig(
    "../publication/figures/pmm_vs_pmmkp.svg", bbox_inches='tight', transparent=False, dpi=500
    )

### Figure 8

fig, axs = plt.subplots(nrows=2, ncols=4, sharey="row", sharex="col", figsize=(linewidth, 3))
fig.subplots_adjust(hspace=0.15)


for i in range(4):
    if i == 0 or i == 3:
        im2=axs[0, i].pcolormesh(params_Bias["dmu"], params_Bias["Vbias"], cond_pmm_single[:,:,i].T, cmap="gist_heat_r", vmin=0, vmax=1, rasterized=True)
        im2=axs[1, i].pcolormesh(params_Bias["dmu"], params_Bias["Vbias"], cond_pmmkp_single[:,:,i].T, cmap="gist_heat_r", vmin=0, vmax=1, rasterized=True)
    else:
        im3=axs[0, i].pcolormesh(params_Bias["dmu"], params_Bias["Vbias"], cond_pmm_single[:,:,i].T, cmap="seismic", vmin=-1, vmax=1, rasterized=True)
        im3=axs[1, i].pcolormesh(params_Bias["dmu"], params_Bias["Vbias"], cond_pmmkp_single[:,:,i].T, cmap="seismic", vmin=-1, vmax=1, rasterized=True)
    axs[1, i].set_xlabel(r"$\epsilon_L/t$")
    axs[1, i].set_xticks([-2,0,2])

axs[0,0].set_ylabel(r"$eV_\text{bias}/t$", labelpad=-5)
axs[0,0].set_yticks([-4,-2,0,2,4])
axs[1,0].set_ylabel(r"$eV_\text{bias}/t$", labelpad=-5)
axs[1,0].set_yticks([-4,-2,0,2,4])

ax_cnl = fig.add_axes([0.3, 0.97, 0.4, 0.025])
ax_cl = fig.add_axes([0.3, 1.12, 0.4, 0.025])

cbar_nl = fig.colorbar(
    im3,
    cax=ax_cnl,
    orientation="horizontal",
    location="top",
    pad=0.15,
    shrink=0.1,
    ticks=[-1,0,1]
)

cbar_l = fig.colorbar(
    im2,
    cax=ax_cl,
    location="top",
    pad=0.75,
    shrink=0.1,
    ticks=[0, 0.5, 1]
)
cbar_nl.ax.set_xlabel(r"$G_{LR}, G_{RL}\, [e^2/h]$")
cbar_nl.ax.xaxis.set_label_coords(1.25, 0)
cbar_nl.ax.get_xaxis().labelpad = 1
cbar_nl.set_ticklabels([r"$-1$", r"$0$", r"$1$"])

cbar_l.ax.set_xlabel(r"$G_{LL}, G_{RR}\,[e^2/h]$")
cbar_l.ax.xaxis.set_label_coords(1.25, 0)
cbar_l.ax.get_yaxis().labelpad = 1
cbar_l.set_ticklabels([r"$0$", r"$0.5$", r"$1$"])


axs[0, 0].set_title(r"$G_{LL}$")
axs[0, 1].set_title(r"$G_{LR}$")
axs[0, 2].set_title(r"$G_{RL}$")
axs[0, 3].set_title(r"$G_{RR}$")

axs[0, 0].text(-2.35,3.1, '(a)', size=12, weight="semibold")
axs[0, 1].text(-2.35,3.1, '(b)', size=12, weight="semibold")
axs[0, 2].text(-2.35,3.1, '(c)', size=12, weight="semibold")
axs[0, 3].text(-2.35,3.1, '(d)', size=12, weight="semibold")
axs[1, 0].text(-2.35,3.1, '(e)', size=12, weight="semibold")
axs[1, 1].text(-2.35,3.1, '(f)', size=12, weight="semibold")
axs[1, 2].text(-2.35,3.1, '(g)', size=12, weight="semibold")
axs[1, 3].text(-2.35,3.1, '(h)', size=12, weight="semibold")


plt.savefig(
    "../publication/figures/full_conductance.svg", bbox_inches='tight', transparent=False, dpi=500
    )

### Full model figures

full_model = np.load("../codes/data/abs_model.npz", allow_pickle=True)

### Figure 9

fig, axs = plt.subplots(1,3, sharey=True, figsize=(linewidth * 0.95, 2))
fig.tight_layout()

for i, panel in enumerate(["(a)", "(b)", "(c)"]):
    im = axs[i].pcolormesh(
        full_model["eps_arr"],
        full_model["eps_arr"],
        full_model["csd"][i],
        cmap="seismic",
        vmin=-0.1,
        vmax=0.1,
        rasterized=True
        )
    axs[i].set_xlabel(r"$\epsilon_R/\Delta_0$")
    axs[i].set_xticks([-0.2, 0.])
    axs[i].text(-0.25, 0.105, panel, size=12, weight="semibold")

axs[0].set_ylabel(r"$\epsilon_L/\Delta_0$")
axs[0].set_yticks([-0.2, 0.])

cax = fig.add_axes(
    [
        axs[2].get_position().x1+0.01,
        axs[2].get_position().y0,0.02,
        axs[2].get_position().height
    ]
)
cbar = plt.colorbar(im, cax=cax)
cbar.ax.set_title(r'$\delta E/\Delta_0$')
plt.savefig(
    "../publication/figures/car_ect_full.svg", bbox_inches='tight', transparent=False, dpi=500
    )

### Figure 10

fig, axs = plt.subplots(nrows=2,ncols=3, sharey="row", figsize=(linewidth*0.95,4))
fig.tight_layout()
fig.subplots_adjust(hspace=0.4)

for i in range(3):
    im1 = axs[0, i].pcolormesh(
        full_model["eps_arr"],
        full_model["eps_arr"],
        full_model["csd_per_U"][i],
        cmap="seismic",
        vmin=-0.1,
        vmax=0.1,
        rasterized=True
)
    axs[0, i].set_xlabel(r"$\epsilon_R/\Delta_0$")

    im2 = axs[1, i].pcolormesh(
        full_model["bias_params"][()]["dmu"],
        full_model["bias_params"][()]["Vbias"],
        full_model["cond_per_U"][i][:,:,0].T,
        cmap="gist_heat_r",
        vmin=0,
        rasterized=True
        )
    axs[1, i].set_xlabel(r"$\delta\epsilon_R/\Delta_0$")

axs[0, 0].set_ylabel(r"$\epsilon_L/\Delta_0$")
axs[1, 0].set_ylabel(r"$eV_\text{bias}/\Delta_0$")
axs[0, 0].set_yticks([-0.2, 0.])

cax1 = fig.add_axes(
    [
        axs[0, 2].get_position().x1+0.02,
        axs[0, 2].get_position().y0,0.02,
        axs[0, 2].get_position().height
    ]
)

cbar1 = plt.colorbar(im1, cax=cax1)
cbar1.ax.set_title(r'$\delta E/\Delta_0$')

cax2 = fig.add_axes(
    [
        axs[1, 2].get_position().x1+0.02,
        axs[1, 2].get_position().y0,0.02,
        axs[1, 2].get_position().height
    ]
)
cbar2 = plt.colorbar(im2, cax=cax2)
cbar2.ax.set_title(r'$G_{LL} [e^2/h]$')

fig.text(0.05, 0.95, '(a)', size=12, weight="semibold")
fig.text(0.36, 0.95, '(b)', size=12, weight="semibold")
fig.text(0.67, 0.95, '(c)', size=12, weight="semibold")
fig.text(0.05, 0.45, '(d)', size=12, weight="semibold")
fig.text(0.36, 0.45, '(e)', size=12, weight="semibold")
fig.text(0.67, 0.45, '(f)', size=12, weight="semibold")

plt.savefig(
    "../publication/figures/u_scaling_full.svg", bbox_inches='tight', transparent=False, dpi=500
    )

### Figure 3

fig1, ax1 = plt.subplots(figsize=(4, 4))

for i in range(3):
    ax1.plot(
        pmm_vs_pmmkp["eps"],
        pmm_vs_pmmkp["spectrum"][:, i:i+1],
        c=color_list[0],
        lw=2,
        alpha=0.8,
        label=r"$p=-1$"
    )
    ax1.plot(
        pmm_vs_pmmkp["eps"],
        pmm_vs_pmmkp["spectrum"][:, 3+i:4+i],
        "--",
        lw=2,
        c=color_list[1],
        alpha=0.8,
        label=r"$p=0$"
        )
    ax1.plot(
        pmm_vs_pmmkp["eps"],
        pmm_vs_pmmkp["spectrum"][:, 6+i:7+i],
        ":",
        lw=2,
        c=color_list[2],
        alpha=0.8,
        label=r"$p=1$"
    )
ax1.set_xlim(-3, 3)
ax1.set_ylim(-3, 3)
ax1.set_xticks([-2,0,2])
ax1.set_yticks([-2,0,2])
ax1.set_xlabel(r"$\epsilon_L/t$")
ax1.set_ylabel(r"$E_i/t$")
handles, labels = ax1.get_legend_handles_labels()
ax1.legend([handles[-3], handles[-2], handles[-1]], [labels[-3], labels[-2], labels[-1]], loc='best')
ax1.set_aspect("equal")

plt.savefig(
    "../publication/figures/pmmkp_spectrum.svg", bbox_inches='tight', transparent=False, dpi=500
    )