from sympy.physics.quantum.fermion import FermionOp
from sympy.physics.quantum import Dagger
from sympy.physics.quantum.state import Ket
import sympy

import matplotlib.pyplot as plt
import scipy.linalg as la


from functools import reduce
from collections import defaultdict
import numpy as np
import scipy.sparse as sp
import re


sigma_0 = np.array([[1, 0],
                    [0, 1]])
sigma_z = np.array([[1,  0],
                    [0, -1]])
sigma_minus = np.array([[0, 1],
                        [0, 0]])

SPINLESS = 0
SPIN_UP = 1
SPIN_DOWN = 2

class fockspace(object):
    def __init__(self, sparse=False):
        self.finalized = False
        self.basis_operators = []
        self.basis_spin = []
        self.op_matrices = None
        self.parity_op = None
        
        if sparse:
            self.kron = lambda a, b: sp.kron(a, b, format='csr')
            self.eye = lambda N: sp.eye(N, format='csr')
        else:
            self.kron = np.kron
            self.eye = np.eye
        
    def add_fermion(self, name, spinful=False):
        start = len(self.basis_operators)
        
 
        def _add_spin_to_name(name):
            if ((m := re.match(r"(.+)_\{(.+)\}$", name)) or
                (m := re.match(r"(.+)_([A-Za-z0-9])$", name)) or
                (m := re.match(r"(.+)_(\\\S+)$", name)) ):
                return (m[1] + "_{" + m[2] + r",\uparrow}",
                        m[1] + "_{" + m[2] + r",\downarrow}")
            else:
                raise ValueError("Cannot add spin to Latex expression '" 
                                 + name + "'")
        
        def _add_single_fermion(name, spinful):
            if spinful:
                name_up, name_down = _add_spin_to_name(name)
                
                self.basis_operators.extend([FermionOp(name_up),
                                             FermionOp(name_down)])
                
                self.basis_spin.extend([SPIN_UP, SPIN_DOWN])
            else:
                self.basis_operators.append(FermionOp(name))
                
                self.basis_spin.append(SPINLESS)

        
        if isinstance(name, str):
            _add_single_fermion(name, spinful)
            
            return self.basis_operators[start:]
        else:
            for n in name:
                _add_single_fermion(n, spinful)

            return self.basis_operators[start:]

    
    def symbolic_vectors(self):
        # This is not optimized at all, but I just want to get something to work
        N = 2**len(self.basis_operators)
        
        self.symbolic_vecs = [None] * N
        
        vacuum_vec = np.zeros(shape=(N,))
        vacuum_vec[0] = 1
        
        sympy_one = sympy.sympify(1)
        
        for i in range(N):
            reverse_bitstring = format(i, 'b')[::-1]
            reverse_bitstring += "0" * (len(self.basis_operators) - len(reverse_bitstring))
            
            # find the index corresponding to the string of operators 
            # defined by `reverse_bitstring`. Actually, it should be 
            # possible to skip this.
            expr = sympy_one
            for index, c in enumerate(reverse_bitstring):
                if c == '1':
                    expr = expr * Dagger(self.basis_operators[index])
            
            vec = self.matrix_numeric(expr) @ vacuum_vec
            ket_index = vec.nonzero()[0][0]
            
            # now make the ket symbol for the vector:
            ket_spec = []
            for index, c in enumerate(reverse_bitstring):
                spin = self.basis_spin[index]
                
                if spin == SPIN_UP:
                    if c == '1': ket_spec.append(r"\uparrow")
                    else: ket_spec.append(r"0")
                elif spin == SPIN_DOWN:
                    if c == '1':
                        if ket_spec[-1] == r"0": ket_spec[-1] = r"\downarrow"
                        else: ket_spec[-1] += r"\downarrow"
                    # note that if down spin is unoccupied, the spin up formatting 
                    # already gives the right string
                else:
                    if c == '1': ket_spec.append("1")
                    else: ket_spec.append("0")
        
            self.symbolic_vecs[ket_index] = Ket(",".join(ket_spec))


    def vec_to_symbolic(self, vec, zero_tol=1e-10):
        
        expr = 0
        
        for index, value in enumerate(vec):
            if abs(value) > zero_tol:
                expr += value * self.symbolic_vecs[index]
                
        return expr
    
    def precompute_operators(self):
        self.op_matrices = {}
        
        N = len(self.basis_operators)
        for i, operator in enumerate(self.basis_operators):
            self.op_matrices[operator] = \
                reduce(self.kron, [ sigma_z ] * i + [ sigma_minus ] +  [self.eye(2**(N-i-1))])
        
        self.finalized = True
    
    def matrix_numeric(self, operator, params=None):
        if params: operator = operator.subs(params)
        
        operator = operator.expand()
        
        if not self.op_matrices: precompute_operators()
        
        N = 2**len(self.basis_operators)
        matrix = 0
        
        for term, coeff in operator.as_coefficients_dict().items():
            term_mat = complex(coeff) * self.eye(N)
            
            for factor in term.as_ordered_factors():
                if factor in self.op_matrices:
                    term_mat = term_mat @ self.op_matrices[factor]
                elif Dagger(factor) in self.op_matrices:
                    term_mat = term_mat @ self.op_matrices[Dagger(factor)].T.conj()
                else:
                    #print(factor)
                    term_mat *= complex(factor)
            
            matrix += term_mat
        
        return matrix
    
    def matrix_function(self, operator, symbolic=False):
        operator = operator.expand()
        
        if not self.op_matrices: precompute_operators()
        
        N = 2**len(self.basis_operators)
        
        terms = defaultdict(complex)
        matrix = 0
        
        for term, coeff in operator.as_coefficients_dict().items():
            #print(term, coeff)
            if symbolic:
                term_mat = coeff * self.eye(N)
            else:
                term_mat = complex(coeff) * self.eye(N)
            
            
            for factor in term.as_ordered_factors():
                if factor in self.op_matrices:
                    term_mat = term_mat @ self.op_matrices[factor]
                elif Dagger(factor) in self.op_matrices:
                    term_mat = term_mat @ self.op_matrices[Dagger(factor)].T.conj()
                else:
                    print(factor)
                    if symbolic:
                        term_mat *= factor
                    else:
                        term_mat *= complex(factor)
                    
            
            matrix += term_mat
        
        return matrix

    def get_all_normal_ordered_operators(self):
        """
        Get all operators c_i^+...c_j^+c_j....c_i that are normal ordered in the sense that
        constants from anticommutators are consistently neglected
        """
        N_max = 2**(2*len(self.basis_operators))
        basis_operator = [Dagger(op) for op in reversed(self.basis_operators)] + self.basis_operators
        n_ops = len(basis_operator)
        
        operator_list = {}
        symbolic_list = []

        for i in range(N_max+1):
            bit_string = [0 for _ in range(n_ops-len(list(map(int, format(i, 'b')))))] + list(map(int, format(i, 'b')))
            symbolic_list.append(np.prod([b_op**bit for b_op, bit in zip(basis_operator, bit_string)]))

        for symbol in symbolic_list:
            operator_list[symbol] = self.matrix_function(symbol)
            
        return operator_list

    def basis_ops_number_rep(self):
        basis_operator = [Dagger(op) for op in reversed(self.basis_operators)] + self.basis_operators
        N_max = 2**(2*len(self.basis_operators))
        n_ops = 2*len(self.basis_operators)
        symbolic_operators = {}
        for i in range(N_max):
            # find partners to collect as number operators
            bit_string = [0 for _ in range(n_ops-len(list(map(int, format(i, 'b')))))] + list(map(int, format(i, 'b')))
            first_half = np.array(bit_string[:n_ops//2])
            second_half = np.array(list(reversed(bit_string[n_ops//2:])))
            where_equal = np.where(first_half+second_half == 2)[0]
            sign = 1
            for index in where_equal:
                first_half[index] = 0
                second_half[index] = 2
                sign *= (-1)**np.sum(np.array(bit_string)[index:n_ops-index])
            new_bit_string = np.array(list(first_half) + list(reversed(second_half)))
    
            #parse backward through second half to count minus signs moving single CAO
            for index in range(n_ops//2):
                if new_bit_string[::-1][index] == 2:
                    sign *= (-1)**np.sum(new_bit_string[::-1][:index]%2)
            # construct operators
            op = np.prod([b_op**bit for b_op, bit in zip(basis_operator, new_bit_string%2)])
            op *= np.prod([sympy.Symbol('n_{}'.format(str(basis_operator[ind].name).split('_')[1]),
                                        commutative=False)
                           for ind in np.where(new_bit_string == 2)[0]])
    
            actual_op = np.prod([b_op**bit for b_op, bit in zip(basis_operator, bit_string)])
            symbolic_operators[sympy.nsimplify(op)] = sign*self.matrix_function(actual_op)
    
        return symbolic_operators

    def get_operator_decomposition(self, operator, get_basis=False):
        '''
        Expand a given operator in terms of basis CAOs.
        '''
        basis_operator_dict = self.get_all_normal_ordered_operators()
        op_list = [k for k,_ in basis_operator_dict.items()]
        mat_list = [v for _,v in basis_operator_dict.items()]
        flat_mat_list = np.array([v.flatten() for v in mat_list]).T

        assert np.allclose(la.det(flat_mat_list), 1)
        assert np.allclose(basis_operator_dict[1].shape, operator.shape)

        if not get_basis:
            return la.solve(flat_mat_list, operator.flatten())
        return la.solve(flat_mat_list, operator.flatten()), basis_operator_dict
    
    def fermionic_representation(self, matrix):
        coefs, ops = self.get_operator_decomposition(matrix, get_basis=True)
        full_op = sum(
            [
                sympy.nsimplify(c)*op
                for c,op in zip(coefs, [b for b in ops.keys()]) 
                if abs(c) > 10**(-5)
            ]
            )
        return full_op

    def get_parity_op(self):
        ops = np.prod([(1-2*Dagger(op)*op) for op in self.basis_operators])
        return self.matrix_function(ops)
        

    def sort_parity_states(self, ham):
        '''
        Split the eigenvectors into odd and even parity sector
        '''

        if np.all(self.parity_op == None):
            self.parity_op = self.get_parity_op()

        par = self.parity_op
        u = la.eigh(par)[1]
        h_p = u.conj().T@ham@u
        e_pbase, v_pbase = la.eigh(h_p)

        par_mat = (u@v_pbase).conj().T@par@(u@v_pbase)
        assert np.allclose(par_mat @ par @ par_mat
                           -np.diag(np.diag(par_mat @ par @ par_mat)),0)

        odd_inds, even_inds = np.where(np.diag(par_mat)<0)[0], np.where(np.diag(par_mat)>0)[0]

        odd_eigs, odd_vecs = e_pbase[odd_inds], u@v_pbase[:,odd_inds]
        even_eigs, even_vecs = e_pbase[even_inds], u@v_pbase[:,even_inds]

        return even_eigs, even_vecs, odd_eigs, odd_vecs