import numpy as np
from sympy.physics.quantum import Dagger
import numpy.linalg as nla
import scipy.optimize

def chemical_potential(c):
    """
    Generates the onsite energy Hamiltonian terms.

    Parameters:
        c (tuple): Tuple containing fermionic operators.

    Returns:
        list: A list containing the terms of the onsite energy Hamiltonian.
    """
    N = int(len(c) / 2)
    mu_hamiltonian = [
        Dagger(c[2 * i]) * c[2 * i] + Dagger(c[2 * i + 1]) * c[2 * i + 1]
        for i in range(N)
    ]

    return mu_hamiltonian

def zeeman_energy(c):
    """
    Generates the Zeeman energy Hamiltonian terms.

    Parameters:
        c (tuple): Tuple containing fermionic operators.

    Returns:
        list: A list containing the terms of the Zeeman energy Hamiltonian.
    """
    N = int(len(c) / 2)
    Ez_ham = [
        Dagger(c[2 * i]) * c[2 * i] - Dagger(c[2 * i + 1]) * c[2 * i + 1]
        for i in range(N)
    ]

    return Ez_ham

def charging_energy(c):
    """
    Generates the charging energy Hamiltonian terms.

    Parameters:
        c (tuple): Tuple containing fermionic operators.

    Returns:
        list: A list containing the terms of the charging energy Hamiltonian.
    """
    N = int(len(c) / 2)
    Ec_ham = [
        Dagger(c[2 * i + 1]) * c[2 * i + 1] * Dagger(c[2 * i]) * c[2 * i]
        for i in range(N)
    ]

    return Ec_ham

def crossed_andreev_reflection(c):
    """
    Generates the crossed Andreev reflection Hamiltonian terms.

    Parameters:
        c (tuple): Tuple containing fermionic operators.

    Returns:
        list: A list containing the terms of the crossed Andreev reflection Hamiltonian.
    """
    N = int(len(c) / 2)
    car_ham = [
        Dagger(c[2 * i]) * Dagger(c[2 * (i + 1) + 1])
        - Dagger(c[2 * i + 1]) * Dagger(c[2 * (i + 1)])
        + Dagger(
            Dagger(c[2 * i]) * Dagger(c[2 * (i + 1) + 1])
            - Dagger(c[2 * i + 1]) * Dagger(c[2 * (i + 1)])
        )
        for i in range(N - 1)
    ]

    return car_ham

def elastic_cotunneling(c):
    """
    Generates the elastic cotunneling Hamiltonian terms.

    Parameters:
        c (tuple): Tuple containing fermionic operators.

    Returns:
        list: A list containing the terms of the elastic cotunneling Hamiltonian.
    """
    N = int(len(c) / 2)
    ect_ham = [
        Dagger(c[2 * i]) * c[2 * (i + 1)]
        + Dagger(c[2 * i + 1]) * c[2 * (i + 1) + 1]
        + Dagger(
            Dagger(c[2 * i]) * c[2 * (i + 1)]
            + Dagger(c[2 * i + 1]) * c[2 * (i + 1) + 1]
        )
        for i in range(N - 1)
    ]

    return ect_ham

def generate_hamiltonian(fs, c, **params):
    """
    Generates the Hamiltonian matrix for a given set of parameters.

    This function constructs the Hamiltonian matrix for a quantum system based on the provided parameters.

    Parameters:
        fs (sparse_fermions.fockspace): An instance of fockspace.
        c (tuple): Tuple containing fermionic operators.
        **params: Variable keyword arguments representing the parameters of the Hamiltonian.
                  Supported parameters include 'mu', 'Ez', 'U', 'Delta', and 't'.

    Returns:
        tuple: A tuple containing two elements:
               - The Hamiltonian matrix representing the quantum system.
               - A dictionary containing matrices for each parameter type, keyed by parameter name.

    Notes:
        - The function constructs matrices for chemical potential (mu), Zeeman energy (Ez), charging energy (U), pairing hopping (Delta), and normal hopping (t).
        - Each matrix is obtained by evaluating the corresponding function (e.g., chemical_potential) with the fermionic operators (c) and converting the resulting operator to a numeric matrix.
        - The Hamiltonian matrix is constructed by summing the product of parameter values and corresponding matrices.
        - The function returns both the Hamiltonian matrix and a dictionary containing all individual parameter matrices.
    """
    mu_mat   = [fs.matrix_numeric(mu) for mu in chemical_potential(c)]
    Delta_mat= [fs.matrix_numeric(delta) for delta in crossed_andreev_reflection(c)]
    Ec_mat   = [fs.matrix_numeric(U) for U in charging_energy(c)]
    t_mat    = [fs.matrix_numeric(t) for t in elastic_cotunneling(c)]
    Ez_mat   = [fs.matrix_numeric(B) for B in zeeman_energy(c)]


    ham_mat = {"mu"     : mu_mat,
                "Ez"    : Ez_mat,
                "U"     : Ec_mat,
                "Delta" : Delta_mat,
                "t"     : t_mat
                }

    matrices = mu_mat + Ez_mat + Ec_mat + Delta_mat + t_mat

    ham = sum(
        [v * matrices[idx] for idx, v in enumerate(params.values())]
    )

    return ham, ham_mat

def remove_double_occupation(fs, matrix, Ec_matrix, return_index=False):
    """
    Removes doubly occupied states from a matrix.

    This function removes the doubly occupied states from a given matrix based on the provided matrix of charging energies.

    Parameters:
        fs (sparse_fermions.fockspace): An instance of fockspace.
        matrix (numpy.ndarray): Matrix from which doubly occupied states will be removed.
        Ec_matrix (numpy.ndarray): Matrix of charging energies.
        return_index (bool, optional): Whether to return the indices of removed states. Defaults to False.

    Returns:
        numpy.ndarray or tuple: If return_index is False, the matrix with doubly occupied states removed.
                                If return_index is True, a tuple containing the matrix with doubly occupied states removed
                                and the indices of removed states.

    Notes:
        - Doubly occupied states are identified based on the diagonal elements of the charging energy matrix (Ec_matrix).
        - The function removes rows and columns corresponding to doubly occupied states from the input matrix.
        - If return_index is True, the function also returns the indices of removed states.

    """
    idx = np.argwhere(np.diag(Ec_matrix) == 0).flatten()
    if return_index:
        return matrix[idx, :][:, idx], idx
    else:
        return matrix[idx, :][:, idx]

def even_odd_separate(ham, P):
    """
    Separates Hamiltonian into even and odd parity sectors.

    This function takes a Hamiltonian matrix and a parity operator matrix and separates the Hamiltonian into even and odd parity sectors.

    Parameters:
        ham (numpy.ndarray): The Hamiltonian matrix.
        P (numpy.ndarray): The parity operator matrix.

    Returns:
        tuple: A tuple containing two matrices representing the Hamiltonian in the even and odd parity sectors, respectively.

    Raises:
        ValueError: If the parity operator has incorrect dimensions or if different parity states mix.

    Notes:
        - The function first sorts the Hamiltonian and the parity operator based on their eigenvalues to ensure consistency.
        - It then separates the Hamiltonian into even and odd parity sectors based on the sorted parity operator.
        - If the even and odd parity sectors are completely decoupled (i.e., there is no mixing between different parity states), the function returns them.
        - If there is mixing between different parity states, a ValueError is raised.

    """
    if P.ndim == 2:
        mask_idx = np.argsort(np.diag(P))
        P_new = np.diag(P)[mask_idx]
    elif P.ndim == 1:
        mask_idx = np.argsort(P)
        P_new = P[mask_idx]
    else:
        raise ValueError("Wrong dimensions for parity operator")
    ham_new = ham[mask_idx, :][:, mask_idx]
    
    ham_odd = ham_new[P_new < 0, :][:, P_new < 0]
    ham_even = ham_new[P_new > 0, :][:, P_new > 0]
    if np.sum(abs(ham_new[P_new < 0,:][:, P_new > 0])) == 0 and np.sum(abs(ham_new[P_new > 0,:][:, P_new < 0])) == 0:
        return ham_even, ham_odd
    else:
        raise ValueError("Different parity states mix")


def generate_dict(mu, Ez, U, Delta, t, N):
    """
    Generates parameters for an effective many-body Hamiltonian with N sites.

    This function constructs a dictionary of parameters for an effective many-body Hamiltonian with N sites. The parameters include onsite energy (mu), Zeeman energy (Ez), charging energy (U), pairing hopping (Delta), and normal hopping (t). Each dot represents a site, connected by both pairing-like and normal hoppings.

    Parameters:
        mu (float): Onsite energy.
        Ez (float): Zeeman energy.
        U (float): Charging energy.
        Delta (float): Pairing hopping.
        t (float): Normal hopping.
        N (int): Number of dots in the system.

    Returns:
        dict: Set of parameters for the effective many-body system, with keys indicating the parameter type followed by the site index.

    """
    params_init = {}
    terms = ["mu", "Ez", "U", "Delta", "t"]
    values = [mu, Ez, U, Delta, t]
    for idx, term in enumerate(terms):
        for i in range(N):
            if i == N-1 and (term == "Delta" or term == "t"):
                pass
            else:
                params_init.update({term+str(i) : values[idx]})
    return params_init


def delta_Egs(ham, P):
    """
    Computes the energy gap between the ground states of different parities.

    This function calculates the energy gap between the lowest eigenvalue with negative parity and the lowest eigenvalue with positive parity of a given Hamiltonian matrix.

    Args:
        ham (numpy.ndarray): The Hamiltonian matrix.
        P (numpy.ndarray): The parity operator matrix.

    Returns:
        float: The energy gap between the ground states of different parities.
    """
    evs, evecs = nla.eigh(ham)
    evecs_parity = np.diag(evecs.conj().T @ P @ evecs).real
    
    ev_o = evs[np.where(evecs_parity < 0)[0][0]]
    ev_e = evs[np.where(evecs_parity > 0)[0][0]]

    return ev_o - ev_e

def find_sweet_spot(ham, P, ham_mat, bounds=[(-1, 1),],
    initial_Delta=0.1,
):
    """
    Finds the sweet spot parameters for the Hamiltonian.

    This function optimizes the parameters of the Hamiltonian to find the 'sweet spot', where the energy gap between the ground states of different parities is minimized.

    Parameters:
        ham (numpy.ndarray): The Hamiltonian matrix.
        P (numpy.ndarray): The parity operator matrix.
        ham_mat (dict): Dictionary containing matrices for each parameter type.
        bounds (list of tuples, optional): Bounds for the optimization parameters. Defaults to [(-1, 1)].
        initial_Delta (float, optional): Initial guess for the pairing hopping parameter Delta. Defaults to 0.1.

    Returns:
        tuple: A tuple containing the optimized values for muLR and Delta.

    Notes:
        - This function utilizes optimization techniques to find the optimal values for muLR and Delta that minimize the energy gap between the ground states of different parities.
        - It first minimizes the energy gap with respect to Delta while keeping muLR fixed.
        - Then, it minimizes the energy gap with respect to muLR while keeping Delta fixed.
        - The resulting values are considered the 'sweet spot' parameters for the Hamiltonian.
    """

    def wrapped_delta_Egs(mus, Delta):
        mu = mus[0]

        ham_mu = ham + mu * sum(ham_mat["mu"]) + Delta * sum(ham_mat["Delta"])
        dEgs = delta_Egs(ham_mu, P)

        return dEgs

    def minimize_Egs(mu):
        Delta = mu[0]
        result = scipy.optimize.minimize(
            wrapped_delta_Egs,
            x0=(0),
            args=(Delta),
        )

        return abs(result.fun)

    result_Delta = scipy.optimize.minimize(
        minimize_Egs,
        x0=(initial_Delta),
        bounds=bounds,
        tol=1e-15,
    )

    def minimize_muLR(mus, Delta):
        mu = mus
        return abs(wrapped_delta_Egs(mu, Delta))

    Delta_ss = result_Delta.x

    result_LR = scipy.optimize.minimize(
        minimize_muLR,
        x0=(0.0),
        args=Delta_ss[0],
        tol=1e-15
    )

    muLR_ss = result_LR.x

    return muLR_ss[0], Delta_ss[0]

def projection(mat, vec):
    """
    Projects a matrix onto subset of vectors.
    Args:
        mat (numpy.ndarray): The square matrix which will be projected.
        vec (numpy.ndarray): The vector that is defined over the subspace.

    Returns:
        numpy.ndarray: Projected matrix.

    Raises:
        AssertionError: If the matrix is not square or if there's a dimensionality mismatch between the matrix and the vector.
    """

    assert mat.shape[0] == mat.shape[1], "Not a square matrix!"
    assert vec.shape[0] == mat.shape[0], "Dimensionality mismatch!"

    if nla.norm((vec.T@mat@vec).imag)<1e-9:
        return (np.round(vec.T@mat@vec, 10).real)
    elif nla.norm((vec.T@mat@vec).real)<1e-9:
        return (1j*np.round(vec.T@mat@vec, 10).imag)
    else:
        return (np.round(vec.T@mat@vec, 10))

def unconstrain_vectors(vec, idx):
    """
    Constructs the full Hilbert space representation from constrained vectors.

    This function takes constrained vectors and constructs the full Hilbert space representation by placing the coefficients of constrained operators of the vecs into the full Hilbert space. The states with doubly occupied terms have zero weight.

    Args:
        vec (numpy.ndarray): The constrained vectors.
        idx (numpy.ndarray): The indices of the states with no double occupation.

    Returns:
        full_hs (numpy.ndarray): The full Hilbert space representation with the coefficients of constrained operators placed appropriately.
    """
    dim1 = vec.shape[0]
    dim2 = int(4**(np.emath.logn(3,dim1)))
    full_hs = np.zeros((dim2,dim1), dtype="complex128")

    # place the coefficients of constrained operators
    # of the sorted basis states to the full Hilbert space.
    # the states with doubly occupied terms have, by construction,
    # zero weight.

    for i, j in enumerate(idx):
        full_hs[j, :] = vec[i, :]

    return full_hs